@@ -5,36 +5,45 @@
 
 from __future__ import annotations
 
+import sys
 from abc import ABC, abstractmethod
+from copy import copy
 from decimal import Decimal
 from io import BytesIO
 from itertools import zip_longest
 from pathlib import Path
 from shutil import copyfileobj
-from typing import Any, BinaryIO, Callable, NamedTuple, Sequence, TypeVar, cast
+from typing import Any, BinaryIO, Callable, NamedTuple, TypeVar, Union, cast
 
 from PIL import Image
 from PIL.ImageCms import ImageCmsProfile
 
-from pikepdf import (
+from pikepdf import jbig2
+from pikepdf._core import Buffer, Pdf, PdfError, StreamDecodeLevel
+from pikepdf._exceptions import DependencyError
+from pikepdf.models import _transcoding
+from pikepdf.objects import (
     Array,
     Dictionary,
     Name,
     Object,
-    Pdf,
-    PdfError,
     Stream,
-    StreamDecodeLevel,
     String,
-    jbig2,
 )
-from pikepdf._exceptions import DependencyError
-from pikepdf._qpdf import Buffer
-from pikepdf._version import __version__
-from pikepdf.models import _transcoding
 
 T = TypeVar('T')
 
+if sys.version_info >= (3, 9):
+    RGBDecodeArray = tuple[float, float, float, float, float, float]
+    GrayDecodeArray = tuple[float, float]
+    CMYKDecodeArray = tuple[float, float, float, float, float, float, float, float]
+    DecodeArray = Union[RGBDecodeArray, GrayDecodeArray, CMYKDecodeArray]
+else:
+    RGBDecodeArray = Any
+    GrayDecodeArray = Any
+    CMYKDecodeArray = Any
+    DecodeArray = Any
+
 
 class UnsupportedImageTypeError(Exception):
     """This image is formatted in a way pikepdf does not supported."""
@@ -53,7 +62,7 @@ class InvalidPdfImageError(Exception):
 
 
 def _array_str(value: Object | str | list):
-    """Simplify pikepdf objects to array of str. Keep Streams and dictionaries intact."""
+    """Simplify pikepdf objects to array of str. Keep streams, dictionaries intact."""
 
     def _convert(item):
         if isinstance(item, (list, Array)):
@@ -72,7 +81,7 @@ def _array_str(value: Object | str | list):
     return result
 
 
-def _ensure_list(value: list[Object] | Dictionary | Array) -> list[Object]:
+def _ensure_list(value: list[Object] | Dictionary | Array | Object) -> list[Object]:
     """Ensure value is a list of pikepdf.Object, if it was not already.
 
     To support DecodeParms which can be present as either an array of dicts or a single
@@ -86,7 +95,7 @@ def _ensure_list(value: list[Object] | Dictionary | Array) -> list[Object]:
 def _metadata_from_obj(
     obj: Dictionary | Stream, name: str, type_: Callable[[Any], T], default: T
 ) -> T | None:
-    """Retrieve metadata from a dictionary or stream, and ensure it is the expected type."""
+    """Retrieve metadata from a dictionary or stream and wrangle types."""
     val = getattr(obj, name, default)
     try:
         return type_(val)
@@ -150,6 +159,29 @@ class PdfImageBase(ABC):
         """List of names of the filters that we applied to encode this image."""
         return self._metadata('Filter', _array_str, [])
 
+    @property
+    def _decode_array(self) -> DecodeArray:
+        """Extract the /Decode array."""
+        decode: list = self._metadata('Decode', _ensure_list, [])
+        if decode and len(decode) in (2, 6, 8):
+            return cast(DecodeArray, tuple(float(value) for value in decode))
+
+        if self.colorspace in ('/DeviceGray', '/CalGray'):
+            return (0.0, 1.0)
+        if self.colorspace == ('/DeviceRGB', '/CalRGB'):
+            return (0.0, 1.0, 0.0, 1.0, 0.0, 1.0)
+        if self.colorspace == '/DeviceCMYK':
+            return (0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0)
+        if self.colorspace == '/ICCBased':
+            if self._approx_mode_from_icc() == 'L':
+                return (0.0, 1.0)
+            if self._approx_mode_from_icc() == 'RGB':
+                return (0.0, 1.0, 0.0, 1.0, 0.0, 1.0)
+
+        raise NotImplementedError(
+            "Don't how to retrieve default /Decode array for image" + repr(self)
+        )
+
     @property
     def decode_parms(self):
         """List of the /DecodeParms, arguments to filters."""
@@ -322,45 +354,12 @@ class PdfImageBase(ABC):
     def as_pil_image(self) -> Image.Image:
         """Convert this PDF image to a Python PIL (Pillow) image."""
 
-    @staticmethod
-    def _remove_simple_filters(obj: Stream, filters: Sequence[str]):
-        """Remove simple lossless compression where it appears.
-
-        Args:
-            obj: the compressed object
-            filters: all files on the data
-        """
-        COMPLEX_FILTERS = {
-            '/DCTDecode',
-            '/JPXDecode',
-            '/JBIG2Decode',
-            '/CCITTFaxDecode',
-        }
-
-        idx = [n for n, item in enumerate(filters) if item in COMPLEX_FILTERS]
-        if idx:
-            if len(idx) > 1:
-                raise NotImplementedError(
-                    f"Object {obj.objgen} has compound complex filters: {filters}. "
-                    "We cannot decompress this."
-                )
-            simple_filters = filters[: idx[0]]
-            complex_filters = filters[idx[0] :]
-        else:
-            simple_filters = filters
-            complex_filters = []
-
-        if not simple_filters:
-            return obj.read_raw_bytes(), complex_filters
-
-        original_filters = obj.Filter
-        try:
-            obj.Filter = Array([Name(s) for s in simple_filters])
-            data = obj.read_bytes(StreamDecodeLevel.specialized)
-        finally:
-            obj.Filter = original_filters
-
-        return data, complex_filters
+    def _repr_png_(self) -> bytes:
+        """Display hook for IPython/Jupyter."""
+        b = BytesIO()
+        with self.as_pil_image() as im:
+            im.save(b, 'PNG')
+            return b.getvalue()
 
 
 class PdfImage(PdfImageBase):
@@ -375,15 +374,19 @@ class PdfImage(PdfImageBase):
 
     obj: Stream
     _icc: ImageCmsProfile | None
+    _pdf_source: Pdf | None
 
-    def __new__(cls, obj):
+    def __new__(cls, obj: Stream):
         """Construct a PdfImage... or a PdfJpxImage if that is what we really are."""
-        instance = super().__new__(cls)
-        instance.__init__(obj)
-        if '/JPXDecode' in instance.filters:
-            instance = super().__new__(PdfJpxImage)
-            instance.__init__(obj)
-        return instance
+        try:
+            # Check if JPXDecode is called for and initialize as PdfJpxImage
+            filters = _ensure_list(obj.Filter)
+            if Name.JPXDecode in filters:
+                return super().__new__(PdfJpxImage)
+        except (AttributeError, KeyError):
+            # __init__ will deal with any other errors
+            pass
+        return super().__new__(PdfImage)
 
     def __init__(self, obj: Stream):
         """Construct a PDF image from a Image XObject inside a PDF.
@@ -464,7 +467,35 @@ class PdfImage(PdfImageBase):
                     ) from e
         return self._icc
 
-    def _extract_direct(self, *, stream: BinaryIO) -> str:
+    def _remove_simple_filters(self):
+        """Remove simple lossless compression where it appears."""
+        COMPLEX_FILTERS = {
+            '/DCTDecode',
+            '/JPXDecode',
+            '/JBIG2Decode',
+            '/CCITTFaxDecode',
+        }
+        indices = [n for n, filt in enumerate(self.filters) if filt in COMPLEX_FILTERS]
+        if len(indices) > 1:
+            raise NotImplementedError(
+                f"Object {self.obj.objgen} has compound complex filters: "
+                f"{self.filters}. We cannot decompress this."
+            )
+        if len(indices) == 0:
+            # No complex filter indices, so all filters are simple - remove them all
+            return self.obj.read_bytes(StreamDecodeLevel.specialized), []
+
+        n = indices[0]
+        if n == 0:
+            # The only filter is complex, so return
+            return self.obj.read_raw_bytes(), self.filters
+
+        obj_copy = copy(self.obj)
+        obj_copy.Filter = Array([Name(f) for f in self.filters[:n]])
+        obj_copy.DecodeParms = Array(self.decode_parms[:n])
+        return obj_copy.read_bytes(StreamDecodeLevel.specialized), self.filters[n:]
+
+    def _extract_direct(self, *, stream: BinaryIO) -> str | None:
         """Attempt to extract the image directly to a usable image file.
 
         If there is no way to extract the image without decompressing or
@@ -500,7 +531,7 @@ class PdfImage(PdfImageBase):
                 )
             return self.mode == 'CMYK' and ct == DEFAULT_CT_CMYK
 
-        data, filters = self._remove_simple_filters(self.obj, self.filters)
+        data, filters = self._remove_simple_filters()
 
         if filters == ['/CCITTFaxDecode']:
             if self.colorspace == '/ICCBased':
@@ -516,7 +547,7 @@ class PdfImage(PdfImageBase):
             stream.write(data)
             return '.jpg'
 
-        raise NotExtractableError()
+        return None
 
     def _extract_transcoded_1248bits(self) -> Image.Image:
         """Extract an image when there are 1/2/4/8 bits packed in byte data."""
@@ -545,7 +576,7 @@ class PdfImage(PdfImageBase):
         return im
 
     def _extract_transcoded_1bit(self) -> Image.Image:
-        if self.mode in ('RGB', 'CMYK'):
+        if not self.image_mask and self.mode in ('RGB', 'CMYK'):
             raise UnsupportedImageTypeError("1-bit RGB and CMYK are not supported")
         try:
             data = self.read_bytes()
@@ -568,7 +599,13 @@ class PdfImage(PdfImageBase):
 
         return im
 
+    def _extract_transcoded_mask(self) -> Image.Image:
+        return self._extract_transcoded_1bit()
+
     def _extract_transcoded(self) -> Image.Image:
+        if self.image_mask:
+            return self._extract_transcoded_mask()
+
         if self.mode in {'DeviceN', 'Separation'}:
             raise HifiPrintImageNotTranscodableError()
 
@@ -609,10 +646,9 @@ class PdfImage(PdfImageBase):
         Returns:
             The file format extension.
         """
-        try:
-            return self._extract_direct(stream=stream)
-        except NotExtractableError:
-            pass
+        direct_extraction = self._extract_direct(stream=stream)
+        if direct_extraction:
+            return direct_extraction
 
         im = None
         try:
@@ -651,10 +687,10 @@ class PdfImage(PdfImageBase):
         Images might be saved as any of .png, .jpg, or .tiff.
 
         Examples:
-            >>> im.extract_to(stream=bytes_io)
+            >>> im.extract_to(stream=bytes_io)  # doctest: +SKIP
             '.png'
 
-            >>> im.extract_to(fileprefix='/tmp/image00')
+            >>> im.extract_to(fileprefix='/tmp/image00')  # doctest: +SKIP
             '/tmp/image00.jpg'
 
         Args:
@@ -697,13 +733,11 @@ class PdfImage(PdfImageBase):
 
         Caller must close the image.
         """
-        try:
-            bio = BytesIO()
-            self._extract_direct(stream=bio)
+        bio = BytesIO()
+        direct_extraction = self._extract_direct(stream=bio)
+        if direct_extraction:
             bio.seek(0)
             return Image.open(bio)
-        except NotExtractableError:
-            pass
 
         im = self._extract_transcoded()
         if not im:
@@ -718,19 +752,28 @@ class PdfImage(PdfImageBase):
 
         if not self.decode_parms:
             raise ValueError("/CCITTFaxDecode without /DecodeParms")
-        if self.decode_parms[0].get("/EncodedByteAlign", False):
-            raise UnsupportedImageTypeError(
-                "/CCITTFaxDecode with /EncodedByteAlign true"
-            )
+
+        expected_defaults = [
+            ("/EncodedByteAlign", False),
+            ("/EndOfLine", False),
+        ]
+        for name, val in expected_defaults:
+            if self.decode_parms[0].get(name, val) != val:
+                raise UnsupportedImageTypeError(
+                    f"/CCITTFaxDecode with decode parameter {name} not equal {val}"
+                )
 
         k = self.decode_parms[0].get("/K", 0)
+        t4_options = None
         if k < 0:
-            ccitt_group = 4  # Pure two-dimensional encoding (Group 4)
+            ccitt_group = 4  # Group 4
         elif k > 0:
             ccitt_group = 3  # Group 3 2-D
+            t4_options = 1
         else:
-            ccitt_group = 2  # Group 3 1-D
-        _black_is_one = self.decode_parms[0].get("/BlackIs1", False)
+            ccitt_group = 3  # Group 3 1-D
+        black_is_one = self.decode_parms[0].get("/BlackIs1", False)
+        decode = self._decode_array
         # PDF spec says:
         # BlackIs1: A flag indicating whether 1 bits shall be interpreted as black
         # pixels and 0 bits as white pixels, the reverse of the normal
@@ -738,36 +781,42 @@ class PdfImage(PdfImageBase):
         # TIFF spec says:
         # use 0 for white_is_zero (=> black is 1) MINISWHITE
         # use 1 for black_is_zero (=> white is 1) MINISBLACK
-        # However, despite the documentation, it seems PDF viewers treat
-        # photometry as 0 when ccitt is involved.
-        # For example see
-        # https://gitlab.gnome.org/GNOME/evince/-/blob/main/backend/tiff/tiff2ps.c#L852-865
-        photometry = 0
+        photometry = 1 if black_is_one else 0
+
+        # If Decode is [1, 0] then the photometry is inverted
+        if len(decode) == 2 and decode == (1.0, 0.0):
+            photometry = 1 - photometry
 
         img_size = len(data)
         if icc is None:
             icc = b''
+
         return _transcoding.generate_ccitt_header(
-            self.size, img_size, ccitt_group, photometry, icc
+            self.size,
+            data_length=img_size,
+            ccitt_group=ccitt_group,
+            t4_options=t4_options,
+            photometry=photometry,
+            icc=icc,
         )
 
     def show(self):  # pragma: no cover
         """Show the image however PIL wants to."""
         self.as_pil_image().show()
 
+    def _set_pdf_source(self, pdf: Pdf):
+        self._pdf_source = pdf
+
     def __repr__(self):
+        try:
+            mode = self.mode
+        except NotImplementedError:
+            mode = '?'
         return (
-            f'<pikepdf.PdfImage image mode={self.mode} '
+            f'<pikepdf.PdfImage image mode={mode} '
             f'size={self.width}x{self.height} at {hex(id(self))}>'
         )
 
-    def _repr_png_(self) -> bytes:
-        """Display hook for IPython/Jupyter."""
-        b = BytesIO()
-        with self.as_pil_image() as im:
-            im.save(b, 'PNG')
-            return b.getvalue()
-
 
 class PdfJpxImage(PdfImage):
     """Support class for JPEG 2000 images. Implements the same API as :class:`PdfImage`.
@@ -790,13 +839,16 @@ class PdfJpxImage(PdfImage):
             and self._jpxpil == other._jpxpil
         )
 
-    def _extract_direct(self, *, stream: BinaryIO):
-        data, filters = self._remove_simple_filters(self.obj, self.filters)
+    def _extract_direct(self, *, stream: BinaryIO) -> str | None:
+        data, filters = self._remove_simple_filters()
         if filters != ['/JPXDecode']:
-            raise UnsupportedImageTypeError(self.filters)
+            return None
         stream.write(data)
         return '.jp2'
 
+    def _extract_transcoded(self) -> Image.Image:
+        return super()._extract_transcoded()
+
     @property
     def _colorspaces(self):
         """Return the effective colorspace of a JPEG 2000 image.
@@ -840,7 +892,7 @@ class PdfJpxImage(PdfImage):
 
 
 class PdfInlineImage(PdfImageBase):
-    """Support class for PDF inline images. Implements the same API as :class:`PdfImage`."""
+    """Support class for PDF inline images."""
 
     # Inline images can contain abbreviations that we write automatically
     ABBREVS = {
@@ -957,7 +1009,7 @@ class PdfInlineImage(PdfImageBase):
             f'size={self.width}x{self.height} at {hex(id(self))}>'
         )
 
-    def _convert_to_pdfimage(self):
+    def _convert_to_pdfimage(self) -> PdfImage:
         # Construct a temporary PDF that holds this inline image, and...
         tmppdf = Pdf.new()
         tmppdf.add_blank_page(page_size=(self.width, self.height))
@@ -968,10 +1020,11 @@ class PdfInlineImage(PdfImageBase):
 
         # ...externalize it,
         tmppdf.pages[0].externalize_inline_images()
-        raw_img = next(im for im in tmppdf.pages[0].images.values())
+        raw_img = cast(Stream, next(im for im in tmppdf.pages[0].images.values()))
 
         # ...then use the regular PdfImage API to extract it.
         img = PdfImage(raw_img)
+        img._set_pdf_source(tmppdf)  # Hold tmppdf open while PdfImage exists
         return img
 
     def as_pil_image(self) -> Image.Image:
